package iotrmi.Java.sample;

import java.io.IOException;
import java.util.Set;
import iotrmi.Java.IoTRMIObject;

import java.util.Arrays;

public class TestClass_Skeleton {

	private String[] methodSignatures = {

		"voidsetA(int)",
		"voidsetB(float)",
		"voidsetC(string)",
		"sumArray(string[])",
		"intsetAndGetA(int)",
		"intsetACAndGetA(string,int)",
		"intcallBack()",
		"voidregisterCallBack(CallBackInterface)"
	};

	private TestClass tc;
	private IoTRMIObject rmiObj;
	private CallBackInterface cbstub;


	/**
	 * Constructors
	 */
	//public TestClass_Skeleton(Object[] paramObj, int _port) throws
	public TestClass_Skeleton(TestClass _tc, int _port) throws
		ClassNotFoundException, InstantiationException,
			IllegalAccessException, IOException {

		//tc = new TestClass((int)paramObj[0], (float)paramObj[1], (String)paramObj[2]);
		tc = _tc;
		rmiObj = new IoTRMIObject(_port, methodSignatures);
	}


	public void waitRequestInvokeMethod() throws IOException {

		// Loop continuously waiting for incoming bytes
		while (true) {

			rmiObj.getMethodBytes();
			String methodSign = rmiObj.getSignature();
			Object[] paramObj = null;
			Object retObj = null;
			System.out.println("Method sign: " + methodSign);

			if (methodSign.equals("voidsetA(int)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { int.class }, 
					new Class<?>[] { null }, new Class<?>[] { null });
				tc.setA((int) paramObj[0]);
			} else if (methodSign.equals("voidsetB(float)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { float.class }, 
					new Class<?>[] { null }, new Class<?>[] { null });
				tc.setB((float) paramObj[0]);
			} else if (methodSign.equals("voidsetC(string)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { String.class }, 
					new Class<?>[] { null }, new Class<?>[] { null });
				tc.setC((String) paramObj[0]);
			} else if (methodSign.equals("sumArray(string[])")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { String[].class }, 
					new Class<?>[] { null }, new Class<?>[] { null });
				retObj = tc.sumArray((String[]) paramObj[0]);
			} else if (methodSign.equals("intsetAndGetA(int)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { int.class }, 
					new Class<?>[] { null }, new Class<?>[] { null });
				retObj = tc.setAndGetA((int) paramObj[0]);
			} else if (methodSign.equals("intsetACAndGetA(string,int)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { String.class, int.class }, 
					new Class<?>[] { null, null }, new Class<?>[] { null, null });
				retObj = tc.setACAndGetA((String) paramObj[0], (int) paramObj[1]);
			} else if (methodSign.equals("voidregisterCallBack(CallBackInterface)")) {
				paramObj = rmiObj.getMethodParams(new Class<?>[] { int.class, String.class, int.class }, 
					new Class<?>[] { null, null, null }, new Class<?>[] { null, null, null });
				CallBackInterface cbstub = new CallBack_Stub((int) paramObj[0], (String) paramObj[1], (int) paramObj[2]);
				tc.registerCallback((CallBackInterface) cbstub);
			} else if (methodSign.equals("intcallBack()")) {
				retObj = tc.callBack();
			} else
				throw new Error("Signature un-recognized!");

			if (retObj != null) {
				rmiObj.sendReturnObj(retObj);
			}
			System.out.println("Servicing remote call for method: " + methodSign);
		}
	}


	public static void main(String[] args) throws Exception {

		int port = 5010;
		TestClass tc = new TestClass(3, 5f, "7911");
		//TestClass_Skeleton tcSkel = new TestClass_Skeleton(new Object[] { 3, 5f, "7911"}, port);
		TestClass_Skeleton tcSkel = new TestClass_Skeleton(tc, port);
		tcSkel.waitRequestInvokeMethod();
	}
}
